<?php
/**
 * ReDJ Community component for Joomla
 *
 * @author selfget.com (info@selfget.com)
 * @package ReDJ
 * @copyright Copyright 2009 - 2012
 * @license GNU Public License
 * @link http://www.selfget.com
 * @version 1.5.7
 */
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();

jimport('joomla.application.component.model');

class ReDJModelItemPages404 extends JModel
{
  /**
   * ItemPages404 id
   *
   * @var int
   */
  var $_id = null;

  /**
   * ItemPages404 data
   *
   * @var array
   */
  var $_data = null;

  /**
   * Constructor
   *
   * @since 1.5
   */
  function __construct()
  {
    parent::__construct();

    $array = JRequest::getVar('cid', array(0), '', 'array');
    $edit  = JRequest::getVar('edit',true);
    if($edit)
      $this->setId((int)$array[0]);
  }

  /**
   * Method to set the itempages404 identifier
   *
   * @access  public
   * @param int itempages404 identifier
   */
  function setId($id)
  {
    // Set itempages404 id and wipe data
    $this->_id   = $id;
    $this->_data = null;
  }

  /**
   * Method to get a itempages404
   *
   * @since 1.5
   */
  function &getData()
  {
    // Load the itempages404 data
    if ($this->_loadData())
    {
      // Initialize some variables
      $user = & JFactory::getUser();
    }
    else  $this->_initData();

    return $this->_data;
  }

  /**
   * Tests if itempages404 is checked out
   *
   * @access  public
   * @param  int  A user id
   * @return  boolean  True if checked out
   * @since  1.5
   */
  function isCheckedOut($uid=0)
  {
    if ($this->_loadData())
    {
      if ($uid) {
        return ($this->_data->checked_out && $this->_data->checked_out != $uid);
      } else {
        return $this->_data->checked_out;
      }
    }
  }

  /**
   * Method to checkin/unlock the itempages404
   *
   * @access  public
   * @return  boolean  True on success
   * @since  1.5
   */
  function checkin()
  {
    if ($this->_id)
    {
      $itempages404 = & $this->getTable();
      if(! $itempages404->checkin($this->_id)) {
        $this->setError($this->_db->getErrorMsg());
        return false;
      }

      return true;
    }
    return false;
  }

  /**
   * Method to checkout/lock the itempages404
   *
   * @access  public
   * @param  int  $uid  User ID of the user checking the item out
   * @return  boolean  True on success
   * @since  1.5
   */
  function checkout($uid = null)
  {
    if ($this->_id)
    {
      // Make sure we have a user id to checkout the item with
      if (is_null($uid)) {
        $user = & JFactory::getUser();
        $uid  = $user->get('id');
      }
      // Lets get to it and checkout the thing...
      $itempages404 = & $this->getTable();
      if(!$itempages404->checkout($uid, $this->_id)) {
        $this->setError($this->_db->getErrorMsg());
        return false;
      }

      return true;
    }
    return false;
  }

  /**
   * Method to store the itempages404
   *
   * @access  public
   * @return  boolean  True on success
   * @since  1.5
   */
  function store($data)
  {
    $row = & $this->getTable();

    // Bind the form fields to the itempages404 table
    if (!$row->bind($data)) {
      $this->setError($this->_db->getErrorMsg());
      return false;
    }

    // Make sure the itempages404 table is valid
    if (!$row->check()) {
      $this->setError($this->_db->getErrorMsg());
      return false;
    }

    // Store the itempages404 table to the database
    if (!$row->store()) {
      $this->setError($this->_db->getErrorMsg());
      return false;
    }

    return true;
  }

  /**
   * Method to remove a itempages404
   *
   * @access  public
   * @return  boolean  True on success
   * @since  1.5
   */
  function delete($cid = array())
  {
    if (count( $cid ))
    {
      JArrayHelper::toInteger($cid);
      $cids = implode(',', $cid);
      $query = 'DELETE FROM #__redj_pages'
        . ' WHERE id IN ( '.$cids.' )';
      $this->_db->setQuery( $query );
      if(!$this->_db->query()) {
        $this->setError($this->_db->getErrorMsg());
        return false;
      }
    }

    return true;
  }

  /**
   * Method to load content itempages404 data
   *
   * @access  private
   * @return  boolean  True on success
   * @since  1.5
   */
  function _loadData()
  {
    // Lets load the content if it doesn't already exist
    if (empty($this->_data))
    {
      $query = 'SELECT * FROM #__redj_pages WHERE id = '.(int) $this->_id;
      $this->_db->setQuery($query);
      $this->_data = $this->_db->loadObject();
      return (boolean) $this->_data;
    }
    return true;
  }

  /**
   * Method to initialise the itempages404 data
   *
   * @access  private
   * @return  boolean  True on success
   * @since  1.5
   */
  function _initData()
  {
    // Lets load the content if it doesn't already exist
    if (empty($this->_data))
    {
      $itempages404 = new stdClass();
      $itempages404->id = 0;
      $itempages404->title = '';
      $itempages404->language = '';
      $itempages404->page = '';
      $itempages404->hits = 0;
      $itempages404->last_visit = 0;
      $itempages404->checked_out = 0;
      $itempages404->checked_out_time = 0;
      $this->_data = $itempages404;
      return (boolean) $this->_data;
    }
    return true;
  }
}
?>
