<?php
/**
 * ReDJ Community component for Joomla
 *
 * @author selfget.com (info@selfget.com)
 * @package ReDJ
 * @copyright Copyright 2009 - 2012
 * @license GNU Public License
 * @link http://www.selfget.com
 * @version 1.5.7
 */
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();

jimport('joomla.application.component.model');

class ReDJModelItemReDJ extends JModel
{
  /**
   * ItemReDJ id
   *
   * @var int
   */
  var $_id = null;

  /**
   * ItemReDJ data
   *
   * @var array
   */
  var $_data = null;

  /**
   * Constructor
   *
   * @since 1.5
   */
  function __construct()
  {
    parent::__construct();

    $array = JRequest::getVar('cid', array(0), '', 'array');
    $edit  = JRequest::getVar('edit',true);
    if($edit)
      $this->setId((int)$array[0]);
  }

  /**
   * Method to set the itemredj identifier
   *
   * @access  public
   * @param int itemredj identifier
   */
  function setId($id)
  {
    // Set itemredj id and wipe data
    $this->_id   = $id;
    $this->_data = null;
  }

  /**
   * Method to get a itemredj
   *
   * @since 1.5
   */
  function &getData()
  {
    // Load the itemredj data
    if ($this->_loadData())
    {
      // Initialize some variables
      $user = &JFactory::getUser();
    }
    else  $this->_initData();

    return $this->_data;
  }

  /**
   * Tests if itemredj is checked out
   *
   * @access  public
   * @param  int  A user id
   * @return  boolean  True if checked out
   * @since  1.5
   */
  function isCheckedOut($uid=0)
  {
    if ($this->_loadData())
    {
      if ($uid) {
        return ($this->_data->checked_out && $this->_data->checked_out != $uid);
      } else {
        return $this->_data->checked_out;
      }
    }
  }

  /**
   * Method to checkin/unlock the itemredj
   *
   * @access  public
   * @return  boolean  True on success
   * @since  1.5
   */
  function checkin()
  {
    if ($this->_id)
    {
      $itemredj = & $this->getTable();
      if(! $itemredj->checkin($this->_id)) {
        $this->setError($this->_db->getErrorMsg());
        return false;
      }

      return true;
    }
    return false;
  }

  /**
   * Method to checkout/lock the itemredj
   *
   * @access  public
   * @param  int  $uid  User ID of the user checking the item out
   * @return  boolean  True on success
   * @since  1.5
   */
  function checkout($uid = null)
  {
    if ($this->_id)
    {
      // Make sure we have a user id to checkout the item with
      if (is_null($uid)) {
        $user = & JFactory::getUser();
        $uid  = $user->get('id');
      }
      // Lets get to it and checkout the thing...
      $itemredj = & $this->getTable();
      if(!$itemredj->checkout($uid, $this->_id)) {
        $this->setError($this->_db->getErrorMsg());
        return false;
      }

      return true;
    }
    return false;
  }

  /**
   * Method to store the itemredj
   *
   * @access  public
   * @return  boolean  True on success
   * @since  1.5
   */
  function store($data)
  {
    $data['case_sensitive'] = isset($data['case_sensitive']) ? 1 : 0;

    $row = & $this->getTable();

    // Bind the form fields to the itemredj table
    if (!$row->bind($data)) {
      $this->setError($this->_db->getErrorMsg());
      return false;
    }

    // if new item, order last in appropriate group
    if (!$row->id) {
      $where = '';
      $row->ordering = $row->getNextOrder( $where );
    }

    // Make sure the itemredj table is valid
    if (!$row->check()) {
      $this->setError($this->_db->getErrorMsg());
      return false;
    }

    // Store the itemredj table to the database
    if (!$row->store()) {
      $this->setError($this->_db->getErrorMsg());
      return false;
    }

    return true;
  }

  /**
   * Method to remove a itemredj
   *
   * @access  public
   * @return  boolean  True on success
   * @since  1.5
   */
  function delete($cid = array())
  {
    if (count( $cid ))
    {
      JArrayHelper::toInteger($cid);
      $cids = implode( ',', $cid );
      $query = 'DELETE FROM #__redj'
        . ' WHERE id IN ( '.$cids.' )';
      $this->_db->setQuery( $query );
      if(!$this->_db->query()) {
        $this->setError($this->_db->getErrorMsg());
        return false;
      }
    }

    return true;
  }

  /**
   * Method to (un)publish a itemredj
   *
   * @access  public
   * @return  boolean  True on success
   * @since  1.5
   */
  function publish($cid = array(), $publish = 1)
  {
    $user = & JFactory::getUser();

    if (count($cid))
    {
      JArrayHelper::toInteger($cid);
      $cids = implode(',', $cid);

      $query = 'UPDATE #__redj'
        . ' SET published = '.(int) $publish
        . ' WHERE id IN ( '.$cids.' )'
        . ' AND ( checked_out = 0 OR ( checked_out = '.(int) $user->get('id').' ) )'
      ;
      $this->_db->setQuery( $query );
      if (!$this->_db->query()) {
        $this->setError($this->_db->getErrorMsg());
        return false;
      }
    }

    return true;
  }

  /**
   * Method to move a itemredj
   *
   * @access  public
   * @return  boolean  True on success
   * @since  1.5
   */
  function move($direction)
  {
    $row = & $this->getTable();
    if (!$row->load($this->_id)) {
      $this->setError($this->_db->getErrorMsg());
      return false;
    }

    if (!$row->move($direction, ' published >= 0 ')) {
      $this->setError($this->_db->getErrorMsg());
      return false;
    }

    return true;
  }

  /**
   * Method to move a redj list
   *
   * @access  public
   * @return  boolean  True on success
   * @since  1.5
   */
  function saveorder($cid = array(), $order)
  {
    $row = & $this->getTable();

    // update ordering values
    for( $i=0; $i < count($cid); $i++ )
    {
      $row->load( (int) $cid[$i] );

      if ($row->ordering != $order[$i])
      {
        $row->ordering = $order[$i];
        if (!$row->store()) {
          $this->setError($this->_db->getErrorMsg());
          return false;
        }
      }
    }

    $row->reorder();

    return true;
  }

  /**
   * Method to set case sensitive property of itemredj
   *
   * @access  public
   * @return  boolean  True on success
   * @since  1.5
   */
  function setcase($cid = array(), $case_sensitive = 1)
  {
    $user = & JFactory::getUser();

    if (count($cid))
    {
      JArrayHelper::toInteger($cid);
      $cids = implode(',', $cid);

      $query = 'UPDATE #__redj'
        . ' SET case_sensitive = '.(int) $case_sensitive
        . ' WHERE id IN ( '.$cids.' )'
        . ' AND ( checked_out = 0 OR ( checked_out = '.(int) $user->get('id').' ) )'
      ;
      $this->_db->setQuery( $query );
      if (!$this->_db->query()) {
        $this->setError($this->_db->getErrorMsg());
        return false;
      }
    }

    return true;
  }

  /**
   * Method to load content itemredj data
   *
   * @access  private
   * @return  boolean  True on success
   * @since  1.5
   */
  function _loadData()
  {
    // Lets load the content if it doesn't already exist
    if (empty($this->_data))
    {
      $query = 'SELECT * FROM #__redj WHERE id = '.(int) $this->_id;
      $this->_db->setQuery($query);
      $this->_data = $this->_db->loadObject();
      return (boolean) $this->_data;
    }
    return true;
  }

  /**
   * Method to initialise the itemredj data
   *
   * @access  private
   * @return  boolean  True on success
   * @since  1.5
   */
  function _initData()
  {
    // Lets load the content if it doesn't already exist
    if (empty($this->_data))
    {
      $itemredj = new stdClass();
      $itemredj->id = 0;
      $itemredj->fromurl = '';
      $itemredj->tourl = null;
      $itemredj->redirect = 301;
      $itemredj->case_sensitive = 1;
      $itemredj->hits = 0;
      $itemredj->last_visit = 0;
      $itemredj->ordering = 0;
      $itemredj->published = 0;
      $itemredj->checked_out = 0;
      $itemredj->checked_out_time = 0;
      $this->_data = $itemredj;
      return (boolean) $this->_data;
    }
    return true;
  }
}
?>
