<?php
/**
 * ReDJ Community component for Joomla
 *
 * @author selfget.com (info@selfget.com)
 * @package ReDJ
 * @copyright Copyright 2009 - 2012
 * @license GNU Public License
 * @link http://www.selfget.com
 * @version 1.5.7
 */

// Check to ensure this file is included in Joomla!  defined('_JEXEC') or die();
jimport('joomla.application.component.model');

class ReDJModelReDJ extends JModel
{
  /**
   * Category data array
   *
   * @var array
   */
  var $_data = null;

  /**
   * Category total
   *
   * @var integer
   */
  var $_total = null;

  /**
   * Pagination object
   *
   * @var object
   */
  var $_pagination = null;

  /**
   * Constructor
   *
   * @since 1.5
   */
  function __construct()
  {
    parent::__construct();

    global $mainframe, $option;

    // Get the request variables
    $limit = $mainframe->getUserStateFromRequest( $option.'.a_limit', 'limit', $mainframe->getCfg('list_limit'), 'int' );
    $limitstart = $mainframe->getUserStateFromRequest( $option.'.a_limitstart', 'limitstart', 0, 'int' );
    $filter_state = $mainframe->getUserStateFromRequest( $option.'.a_filter_state', 'filter_state', '', 'word' );
    $filter_order = $mainframe->getUserStateFromRequest( $option.'.a_filter_order', 'filter_order', 'a.ordering', 'cmd' );
    $filter_order_Dir = $mainframe->getUserStateFromRequest( $option.'.a_filter_order_Dir', 'filter_order_Dir', 'asc', 'word' );
    $search = $mainframe->getUserStateFromRequest( $option.'.a_search', 'search', '', 'string' );

    // Convert search to lower case
    $search = JString::strtolower( $search );

    // in case limit has been changed, adjust limitstart accordingly
    $limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

    // set model vars
    $this->setState('limit', $limit);
    $this->setState('limitstart', $limitstart);
    $this->setState('filter_state', $filter_state);
    $this->setState('filter_order', $filter_order);
    $this->setState('filter_order_Dir', $filter_order_Dir);
    $this->setState('search', $search);
  }

  /**
   * Method to get redj item data
   *
   * @access public
   * @return array
   */
  function getData()
  {
     // Lets load the content if it doesn't already exist
     if (empty($this->_data))
     {
       $query = $this->_buildQuery();
       $this->_data = $this->_getList($query, $this->getState('limitstart'), $this->getState('limit'));
     }

     return $this->_data;
  }

  /**
   * Method to get the total number of redj items
   *
   * @access public
   * @return integer
   */
  function getTotal()
  {
    // Lets load the content if it doesn't already exist
    if (empty($this->_total))
    {
      $query = $this->_buildQuery();
      $this->_total = $this->_getListCount($query);
    }

    return $this->_total;
  }

  /**
   * Method to get a pagination object for the redj
   *
   * @access public
   * @return integer
   */
  function getPagination()
  {
    // Lets load the content if it doesn't already exist
    if (empty($this->_pagination))
    {
      jimport('joomla.html.pagination');
      $this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
    }

    return $this->_pagination;
  }

  function _buildQuery()
  {
    // Get the WHERE and ORDER BY clauses for the query
    $where = $this->_buildContentWhere();
    $orderby = $this->_buildContentOrderBy();

    $query = ' SELECT a.id, a.fromurl, a.tourl, a.redirect, a.case_sensitive, a.hits, '
      . 'a.last_visit, a.ordering, a.published, a.checked_out '
      . ' FROM #__redj AS a '
      . $where
      . $orderby
    ;

    return $query;
  }

  function _buildContentOrderBy()
  {
    $filter_order = $this->getState('filter_order');
    $filter_order_Dir = $this->getState('filter_order_Dir');

    if ($filter_order == 'a.ordering'){
      $orderby = ' ORDER BY a.ordering '.$filter_order_Dir;
    } else {
      $orderby = ' ORDER BY '.$filter_order.' '.$filter_order_Dir.' , a.ordering';
    }

    return $orderby;
  }

  function _buildContentWhere()
  {
    $filter_state = $this->getState('filter_state');
    $search = $this->getState('search');

    $where = array();

    if ($search) {
      $where[] = 'LOWER(a.fromurl) LIKE '.$this->_db->Quote('%'.$search.'%').'OR LOWER(a.tourl) LIKE '.$this->_db->Quote('%'.$search.'%');
    }
    if ( $filter_state ) {
      if ( $filter_state == 'P' ) {
     $where[] = 'a.published = 1';
      } else if ($filter_state == 'U' ) {
     $where[] = 'a.published = 0';
      }
    }

    $where  = ( count( $where ) ? ' WHERE '. implode( ' AND ', $where ) : '' );

    return $where;
  }
}
?>
