<?php
/**
 * ReDJ Community plugin for Joomla
 *
 * @author selfget.com (info@selfget.com)
 * @package ReDJ
 * @copyright Copyright 2009 - 2012
 * @license GNU Public License
 * @link http://www.selfget.com
 * @version 1.5.7
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');

define('VARCHAR_SIZE', 255);

function strleft($s1, $s2) {
  return substr($s1, 0, strpos($s1, $s2));
}

function getPrefix() {
  $s = empty($_SERVER['HTTPS']) ? '' : ($_SERVER['HTTPS'] == "on") ? 's' : '';
  $protocol = strleft(strtolower($_SERVER['SERVER_PROTOCOL']), "/") . $s;
  $port = ($_SERVER['SERVER_PORT'] == '80') ? '' : (':' . $_SERVER['SERVER_PORT']);
  return $protocol . '://' . $_SERVER['SERVER_NAME'] . $port;
}

function selfURL() {
  return getPrefix() . $_SERVER['REQUEST_URI'];
}

function getHost($address) {
  $parseUrl = parse_url(trim($address));
  return @trim($parseUrl['host'] ? $parseUrl['host'] : array_shift(explode('/', $parseUrl['path'], 2)));
}

/*
 * An improved version of JURI::base()
 */
function getBase($pathonly = false) {
  if ( (strpos(php_sapi_name(), 'cgi') !== false) && (!ini_get('cgi.fix_pathinfo')) && (strlen($_SERVER['REQUEST_URI']) > 0) ) {
    //Apache CGI
    if (strlen(trim($_SERVER['PATH_INFO'])) > 0) {
      $p = strrpos($_SERVER['PHP_SELF'], $_SERVER['PATH_INFO']);
      if ($p !== false) { $s = substr($_SERVER['PHP_SELF'], 0, $p); }
    } else {
      $p = $_SERVER['PHP_SELF'];
    }
    $base_path = trim(rtrim(dirname(str_replace(array('"', '<', '>', "'"), '', $p)), '/\\'));
    // Check if base path was correctly detected, or use another method
    /*
       On some Apache servers (mainly using cgi-fcgi) it happens that the base path is not correctly detected.
       For URLs like http://www.site.com/index.php/content/view/123/5 the server returns a wrong PHP_SELF variable.

       WRONG:
       [REQUEST_URI] => /index.php/content/view/123/5
       [PHP_SELF] => /content/view/123/5

       CORRECT:
       [REQUEST_URI] => /index.php/content/view/123/5
       [PHP_SELF] => /index.php/content/view/123/5

       And this lead to a wrong result for JURI::base function.

       WRONG:
       JURI::base(true) => /content/view/123
       JURI::base(false) => http://www.site.com/content/view/123/

       CORRECT:
       getBase(true) =>
       getBase(false):http://www.site.com/
    */
    if (strlen($base_path) > 0) {
      if (strpos($_SERVER['REQUEST_URI'], $base_path) !== 0) {
        $base_path = trim(rtrim(dirname($_SERVER['SCRIPT_NAME']), '/\\'));
      }
    }
  } else {
    //Others
    $base_path = trim(rtrim(dirname($_SERVER['SCRIPT_NAME']), '/\\'));
  }

  return $pathonly === false ? getPrefix() . $base_path . '/' : $base_path;
}

function getMacroParamValue($macro, $param) {
  static $vars = array();
  static $paths = array();

  $value = '';
  switch ($macro) {
    case 'setvars':
      $vars = $param;
      break;
    case 'setpaths':
      $paths = $param;
      break;
    case 'querybuild':
    case 'querybuildfull':
      $build_vars = explode(',', $param);
      foreach ($build_vars as $k => $v) {
        $p = strpos($v, "=");
        if ($p === false) {
          // Only parameter name
          if (isset($vars[$v])) { // Need to check only not-null values
            $value .= (($value === '') ? '' : '&') . $v . '=' . $vars[$v];
          }
        } else {
          // New parameter or overrides existing
          $pn = substr($v, 0, $p);
          $pv = substr($v, $p + 1, strlen($v) - $p - 1);
          if ((strlen($pn) > 0) && (strlen($pv) > 0)) { // Need to take only not-null names and values
            $value .= (($value === '') ? '' : '&') . $pn . '=' . $pv;
          }
        }
      }
      if ( ($macro === 'querybuildfull') && ($value !== '') ) { $value = '?' . $value; }
      break;
    case 'queryvar':
      @$value = $vars[$param];
      break;
    case 'pathltrim':
      $value = $paths['path'];
      if (strpos($value, $param) === 0) {
        $value = substr($value, strlen($param), strlen($value) - strlen($param));
      }
      break;
    case 'pathrtrim':
      $value = $paths['path'];
      if (strpos($value, $param) === (strlen($value) - strlen($param))) {
        $value = substr($value, 0, strlen($value) - strlen($param));
      }
      break;
  }
  return $value;
}

/**
 *
 * Truncate the URL if its lenght exceed the max number of characters (UTF-8) in the varchar definition
 *
 * @param string $url The URL to sanitize
 * @since 1.5
 */
function sanitizeURL($url) {
  //return utf8_encode(substr(utf8_decode($url), 0, VARCHAR_SIZE));
  return mb_substr($url, 0, VARCHAR_SIZE, 'UTF-8');
}

class plgSystemReDJ extends JPlugin {

/**
*
* @var string
* @access  private
*/
var $_visited_url = '';

/**
*
* @var string
* @access  private
*/
var $_siteurl = '';

/**
*
* @var string
* @access  private
*/
var $_baseonly = '';

/**
*
* @var string
* @access  private
*/
var $_tourl = '';

/**
*
* @var integer
* @access  private
*/
var $_redirect = 301;

  /**
   * Constructor
   *
   * For php4 compatability we must not use the __constructor as a constructor for plugins
   * because func_get_args ( void ) returns a copy of all passed arguments NOT references.
   * This causes problems with cross-referencing necessary for the observer design pattern.
   *
   * @param object $subject The object to observe
   * @since 1.5
   */
  function plgSystemReDJ(& $subject, $config) {
    parent :: __construct($subject, $config);
    // Save the current error handler
    $GLOBALS["_REDJ_JERROR_HANDLER"] = JError::getErrorHandling(E_ERROR);
    if ( !( isset($GLOBALS["_REDJ_JERROR_HANDLER"]["mode"]) && isset($GLOBALS["_REDJ_JERROR_HANDLER"]["options"]["0"]) && isset($GLOBALS["_REDJ_JERROR_HANDLER"]["options"]["1"]) ) ) {
      $GLOBALS["_REDJ_JERROR_HANDLER"] = array("mode" => "callback", "options" => array("0" => "JError", "1" => "customErrorPage"));
    }
    // Set new error handler
    JError::setErrorHandling(E_ERROR, 'callback', array('plgSystemReDJ', 'custom_error'));
    $this->_visited_url = selfURL();
    $this->_siteurl = str_replace( 'https://', '', getBase() );
    $this->_siteurl = rtrim ( str_replace( 'http://', '', $this->_siteurl ), '/');
    $this->_baseonly = getBase(true);
  }

  function custom_error(& $object) {
    // This is a static method that will be called as a callback function (plgSystemReDJ::custom_error())
    // So, no $this reference to object instance can be used

    global $mainframe;

    if (JError::isError($object) == true) {
      $plugin = & JPluginHelper :: getPlugin('system', 'redj');
      $params = new JParameter($plugin->params);
      $customerror404page = $params->get('customerror404page', 0);
      $error404pageid = $params->get('error404pageid', "");
      $trackerrors = $params->get('trackerrors', 0);
      $redirectallerrors = $params->get('redirectallerrors', 0);
      $redirectallerrorsurl = $params->get('redirectallerrorsurl', "");
      $db = JFactory::getDBO();

      if ($trackerrors) {
        // Track error URL calls
        $visited_url = selfURL();
        $error_code = $object->code;
        $last_visit = date("Y-m-d H:i:s");
        $last_referer = @$_SERVER['HTTP_REFERER'];
        $db->setQuery("INSERT INTO #__redj_errors (`visited_url`, `error_code`, `hits`, `last_visit`, `last_referer`) VALUES (" . $db->quote( $visited_url ) . ", " . $db->quote( $error_code ) . ", 1, " . $db->quote( $last_visit ) . ", " . $db->quote( $last_referer ) . ") ON DUPLICATE KEY UPDATE `hits` = `hits` + 1, `last_visit` = " . $db->quote( $last_visit ) . ", `last_referer` = " . $db->quote( $last_referer ));
        $res = @$db->query();
      }

      if ( ($redirectallerrors) && (!empty($redirectallerrorsurl)) ){
        $siteurl = str_replace( 'https://', '', getBase() );
        $siteurl = rtrim ( str_replace( 'http://', '', $siteurl ), '/');
        $tourl = str_replace( '{siteurl}', $siteurl, $redirectallerrorsurl );
        @ob_end_clean();
        header("HTTP/1.1 301 Moved Permanently");
        header('Location: ' . $tourl);
        exit;
      }

      $db->setQuery("SELECT * FROM #__redj_pages WHERE id=" . $db->quote( $error404pageid ));
      $items = $db->loadObjectList();

      if (($object->code == 404) && ($customerror404page) && (count($items) > 0) && (!empty($items[0]->page))) { // Custom error handler
        // Update hits
        $last_visit = date("Y-m-d H:i:s");
        $db->setQuery("UPDATE #__redj_pages SET hits = hits + 1, last_visit = " . $db->quote( $last_visit ) . " WHERE id = " . $db->quote( $error404pageid ));
        $res = @$db->query();
        // Initialize variables
        jimport('joomla.document.document');
        $app       = & JFactory::getApplication();
        $document  = & JDocument::getInstance('error');
        $config    = & JFactory::getConfig();

        // Get the current template from the application
        $template = $app->getTemplate();

        // Push the error object into the document
        $document->setError($object);

        @ob_end_clean();
        // Call default render to set header
        $data = $document->render(false, array (
          'template' => $template,
          'directory' => JPATH_THEMES,
          'debug' => $config->getValue('config.debug')
        ));
        // Replace macros in custom body
        $custombody = $items[0]->page;
        $siteurl = str_replace( 'https://', '', getBase() );
        $siteurl = rtrim ( str_replace( 'http://', '', $siteurl ), '/');
        $custombody = str_replace( '{siteurl}', $siteurl, $custombody );
        $sitename = $mainframe->getCfg('sitename');
        $custombody = str_replace( '{sitename}', $sitename, $custombody );
        $sitemail = $mainframe->getCfg('mailfrom');
        $custombody = str_replace( '{sitemail}', $sitemail, $custombody );
        $errormessage = $object->message;
        $custombody = str_replace( '{errormessage}', $errormessage, $custombody );
        // Also replace {article} macro
        $regex_base = '\{(article)\s+(\d+)\}';
        $regex_search = "/(.*?)$regex_base(.*)/si";
        $regex_replace = "/$regex_base/si";
        $found = preg_match( $regex_search, $custombody, $matches );
        while ($found) {
          $content_id = $matches[3];
          $content_query = "SELECT `introtext`, `fulltext` FROM #__content WHERE `id`=".(int)$content_id;
          $content_db = & JFactory::getDBO();
          $content_db->setQuery( $content_query );
          $content_row = $content_db->loadObject();
          if (is_object($content_row)) {
            $content = $content_row->introtext . $content_row->fulltext;
          } else {
            $content = '';
          }
          $custombody = preg_replace( $regex_replace, $content, $custombody ); // Replace all occurrences
          $found = preg_match( $regex_search, $custombody, $matches );
        }

        // Set the custom body
        JResponse::setBody($custombody);
        echo JResponse::toString();
        @ob_flush();
      }

      // Restore saved error handler
      JError::setErrorHandling(E_ERROR, $GLOBALS["_REDJ_JERROR_HANDLER"]["mode"], array($GLOBALS["_REDJ_JERROR_HANDLER"]["options"]["0"], $GLOBALS["_REDJ_JERROR_HANDLER"]["options"]["1"]));
      // Re-raise original error...
      JError::raise($object->level, $object->code, $object->message, $object->info, $object->backtrace);

    }
  }

  function setRedirect() {

    $db = JFactory::getDBO();
    $currenturi = utf8_decode(urldecode($_SERVER['REQUEST_URI']));

    // Remove the base path from the current URI
    $plugin = & JPluginHelper :: getPlugin('system', 'redj');
    $params = new JParameter($plugin->params);
    $basepath = trim($params->get('basepath', ''), ' /');
    if ($basepath != '') {
      if (strpos($currenturi, '/'.$basepath.'/') === 0) {
        $currenturi = substr($currenturi, strlen($basepath) + 1);
      }
    }

    $db->setQuery("SELECT * FROM #__redj WHERE ( ((" . $db->quote( $currenturi ) . " REGEXP BINARY fromurl)>0 and case_sensitive<>'0') or ((" . $db->quote( $currenturi ) . " REGEXP fromurl)>0 and case_sensitive='0') ) and published='1' ORDER BY ordering");
    $items = $db->loadObjectList();
    if ( count($items) > 0 ) {
      // Notes: if more than one match with current URI, we takes only the first one with ordering set
      if ( !empty($items[0]->tourl) ) {
        // Update hits
        $last_visit = date("Y-m-d H:i:s");
        $db->setQuery("UPDATE #__redj SET hits = hits + 1, last_visit = " . $db->quote( $last_visit ) . " WHERE id = " . $db->quote( $items[0]->id ));
        $res = @$db->query();

        // Initialize macro related variables
        getMacroParamValue('seturl', $this->_visited_url);
        $uri_visited_url =& JURI::getInstance($this->_visited_url);
        getMacroParamValue('setvars', $uri_visited_url->_vars);
        $authority = (isset($uri_visited_url->_port)) ? $uri_visited_url->_host . ':' . $uri_visited_url->_port : $uri_visited_url->_host;
        $authority = (isset($uri_visited_url->_user)) ? $uri_visited_url->_user . ':' . $uri_visited_url->_pass . '@' . $authority : $authority;
        $pathfrombase = (strlen($this->_baseonly) > 0) ? substr($uri_visited_url->_path, strlen($this->_baseonly), strlen($uri_visited_url->_path) - strlen($this->_baseonly)) : $uri_visited_url->_path;
        getMacroParamValue('setpaths', array('path' => $uri_visited_url->_path, 'pathfrombase' => $pathfrombase));

        /*
          http://fredbloggs:itsasecret@www.example.com:8080/path/to/Joomla/section/cat/index.php?task=view&id=32#anchorthis
          \__/   \________/ \________/ \_____________/ \__/\___________________________________/ \_____________/ \________/
           |          |         |              |        |                   |                           |             |
         scheme     user       pass          host      port                path                       query       fragment

        Supported macro for destination URL:
           {siteurl}                                                    www.example.com/path/to/Joomla
           {scheme}                                                     http
           {host}                                                       www.example.com
           {port}                                                       8080
           {user}                                                       fredbloggs
           {pass}                                                       itsasecret
           {path}                                                       /path/to/Joomla/section/cat/index.php
           {query}                                                      task=view&id=32
           {queryfull}                                                  ?task=view&id=32
           {querybuild id,task}                                         id=32&task=view
           {querybuild id,task=edit}                                    id=32&task=edit
           {querybuild id,task=view,ItemId=12}                          id=32&task=view&ItemId=12
           {querybuildfull id,task}                                     ?id=32&task=view
           {querybuildfull id,task=save}                                ?id=32&task=save
           {querybuildfull id,task,action=close}                        ?id=32&task=view&action=close
           {querydrop task}                                             id=32
           {querydrop id,task=edit}                                     task=edit
           {querydrop id,task=save,action=close}                        task=save&action=close
           {querydropfull task}                                         ?id=32
           {querydropfull id,task=save}                                 ?task=save
           {querydropfull id,task=edit,action=close}                    ?task=edit&action=close
           {queryvar task}                                              view
           {queryvar id}                                                32
           {authority}                                                  fredbloggs:itsasecret@www.example.com:8080
           {baseonly}                                                   /path/to/Joomla (empty when installed on root, i.e. it will never contains a trailing slash)
           {pathfrombase}                                               /section/cat/index.php
           {pathltrim /path/to}                                         /Joomla/section/cat/index.php
           {pathrtrim /index.php}                                       /path/to/Joomla/section/cat
           {pathfrombaseltrim /section}                                 /cat/index.php
           {pathfrombasertrim index.php}                                /section/cat/
           {preg_match N}pattern{/preg_match}                           (return the N-th matched pattern on the full source url, where N = 0 when not specified)
           {preg_match}/([^\/]+)(\.php|\.html|\.htm)/i{/preg_match}     index.php
           {preg_match 2}/([^\/]+)(\.php|\.html|\.htm)/i{/preg_match}   .php
           {preg_select table,column,key,N}pattern{/preg_select}        (uses the N-th matched result to execute a SQL query (SELECT column FROM table WHERE key = matchN). Support #__ notation for table name)
        */

        // Supported macros patterns
        $patterns = array();
        $patterns[0] = "/\{siteurl\}/";
        $patterns[1] = "/\{scheme\}/";
        $patterns[2] = "/\{host\}/";
        $patterns[3] = "/\{port\}/";
        $patterns[4] = "/\{user\}/";
        $patterns[5] = "/\{pass\}/";
        $patterns[6] = "/\{path\}/";
        $patterns[7] = "/\{query\}/";
        $patterns[8] = "/\{queryfull\}/";
        $patterns[9] = "/\{querybuild ([^\}]+)\}/eU";
        $patterns[10] = "/\{querybuildfull ([^\}]+)\}/eU";
        $patterns[13] = "/\{queryvar ([^\}]+)\}/eU";
        $patterns[14] = "/\{authority\}/";
        $patterns[15] = "/\{baseonly\}/";
        $patterns[16] = "/\{pathfrombase\}/";
        $patterns[17] = "/\{pathltrim ([^\}]+)\}/eU";
        $patterns[18] = "/\{pathrtrim ([^\}]+)\}/eU";

        // Supported macros replacements
        $replacements = array();
        $replacements[0] = $this->_siteurl;
        $replacements[1] = $uri_visited_url->_scheme;
        $replacements[2] = $uri_visited_url->_host;
        $replacements[3] = $uri_visited_url->_port;
        $replacements[4] = $uri_visited_url->_user;
        $replacements[5] = $uri_visited_url->_pass;
        $replacements[6] = $uri_visited_url->_path;
        $replacements[7] = $uri_visited_url->_query;
        $replacements[8] = (isset($uri_visited_url->_query)) ? '?' . $uri_visited_url->_query : '';
        $replacements[9] = "getMacroParamValue('querybuild', '\\1')";
        $replacements[10] = "getMacroParamValue('querybuildfull', '\\1')";
        $replacements[13] = "getMacroParamValue('queryvar', '\\1')";
        $replacements[14] = $authority;
        $replacements[15] = $this->_baseonly;
        $replacements[16] = $pathfrombase;
        $replacements[17] = "getMacroParamValue('pathltrim', '\\1')";
        $replacements[18] = "getMacroParamValue('pathrtrim', '\\1')";

        $this->_tourl = preg_replace($patterns, $replacements, $items[0]->tourl);
        $this->_redirect = $items[0]->redirect;
      }
    }

  }

  function onAfterInitialise() {
    global $mainframe;

    // Only mod site pages that are html docs (no admin, install, etc.)
    if ( !$mainframe->isSite() ) return;
    if ( JRequest::getBool('no_html') === true ) return;

    $plugin = & JPluginHelper :: getPlugin('system', 'redj');
    $params = new JParameter($plugin->params);
    $trackreferers = $params->get('trackreferers', 0);

    $db = JFactory::getDBO();

    if ($trackreferers) {
      // Track referers URL calls
      $excludereferers = preg_split("/[\r\n]+/", $params->get('excludereferers', ''));
      foreach ($excludereferers as $key => $value) {
        if (trim($value) == '') {
          // Remove blanks
          unset($excludereferers[$key]);
        } else {
          // Replace macros
          if ($value == '{self}') { $excludereferers[$key] = $_SERVER['SERVER_NAME']; }
          if ($value == '{none}') { $excludereferers[$key] = ''; }
        }
      }
      $referer_url = @$_SERVER['HTTP_REFERER'];
      $domain = getHost($referer_url);
      if ( !in_array($domain, $excludereferers) ) {
        $last_visit = date("Y-m-d H:i:s");

        $db->setQuery("INSERT IGNORE INTO `#__redj_visited_urls` (`id`, `visited_url`) VALUES (NULL, " . $db->quote( $this->_visited_url ) . ")");
        $res = @$db->query();

        $db->setQuery("INSERT IGNORE INTO `#__redj_referer_urls` (`id`, `referer_url`, `domain`) VALUES (NULL, " . $db->quote( $referer_url ) . ", " . $db->quote( $domain ) . ")");
        $res = @$db->query();

        $db->setQuery("INSERT INTO `#__redj_referers` (`id`, `visited_url`, `referer_url`, `hits`, `last_visit`) VALUES (NULL, (SELECT `id` FROM `#__redj_visited_urls` WHERE `visited_url` = " . $db->quote( sanitizeURL( $this->_visited_url ) ) . "), (SELECT `id` FROM `#__redj_referer_urls` WHERE `referer_url` = " . $db->quote( sanitizeURL( $referer_url ) ) . "), 1, " . $db->quote( $last_visit ) . " ) ON DUPLICATE KEY UPDATE `hits` = `hits` + 1, `last_visit` = " . $db->quote( $last_visit ));
        $res = @$db->query();
      }
    }

    $this->setRedirect();

    if ( !empty($this->_tourl) ) {
      @ob_end_clean();
      switch ($this->_redirect) {
        case 307:
          header("HTTP/1.1 307 Temporary Redirect");
          header('Location: ' . $this->_tourl);
          exit;
        default:
          header("HTTP/1.1 301 Moved Permanently");
          header('Location: ' . $this->_tourl);
          exit;
      }
    }

  }

}
